import SwiftUI
import Collections
import HealthKit
import HealthDB
import HealthKitExtensions
import CoreLocation

struct WorkoutDetailView: View {

    @EnvironmentObject
    var database: Database

    private let store = HealthStore()

    let workout: Workout

    @State
    private var healthWorkout: HKWorkout?

    @State
    var heartRateSamplesInHealth: [HeartRate] = []

    @State
    var heartRateSamplesInDatabase: [HeartRate] = []

    @State
    var locationSamples: [CLLocation] = []

    var body: some View {
        List {
            Section("Info") {
                DetailRow("ID", value: workout.id)
                DetailRow("Total Distance", kilometer: workout.totalDistance)
                DetailRow("Duration", duration: workout.duration)
                DetailRow("Goal", value: workout.goal)
            }
            if !workout.workoutActivities.isEmpty {
                Section("Activities") {
                    ForEach(workout.workoutActivities, id: \.startDate) { activity in
                        NavigationLink(value: activity) {
                            DetailRow(activity.workoutConfiguration.activityType.description,
                                      date: activity.startDate)
                        }
                    }
                }
            }
            if !workout.workoutEvents.isEmpty {
                Section("Events") {
                    NavigationLink(value: workout.workoutEvents) {
                        DetailRow("Events", value: workout.workoutEvents.count)
                    }
                }
            }
            if !workout.metadata.isEmpty {
                Section("Metadata") {
                    NavigationLink {
                        WorkoutMetadataView(metadata: workout.metadata)
                    } label: {
                        DetailRow("Metadata", value: workout.metadata.count)
                    }
                }
            }
            Section("Heart Rate") {
                DetailRow("Samples", value: "\(heartRateSamplesInDatabase.count)")
                DetailRow("Range", value: "\(heartRateSamplesInDatabase.minimumHeartRate) - \(heartRateSamplesInDatabase.maximumHeartRate)")
            }
            if let healthWorkout {
                Section("Matching health workout") {
                    DetailRow("Duration", value: healthWorkout.duration.durationString)
                    DetailRow("Distance", kilometer: healthWorkout.distance?.doubleValue(for: .meterUnit(with: .kilo)))
                    DetailRow("Heart rate samples", value: "\(heartRateSamplesInHealth.count)")
                    DetailRow("Heart rate range", value: "\(heartRateSamplesInHealth.minimumHeartRate) - \(heartRateSamplesInHealth.maximumHeartRate)")
                }
            }


            if !locationSamples.isEmpty {
                Section("Locations") {
                    DetailRow("Count", value: "\(locationSamples.count)")
                }
            }
        }
        .navigationTitle(workout.typeString)
        .navigationDestination(for: HKWorkoutActivity.self) { activity in
            ActivityDetailView(workout: workout, activity: activity)
        }
        .navigationDestination(for: [HKWorkoutEvent].self) {
            WorkoutEventsView(events: $0)
        }
        .onAppear(perform: loadSamples)
    }

    private func loadSamples() {
        Task {
            checkPermissionsAndSearchHealth()
            do {
                guard let samples: [HeartRate] = try database.store?.samples(associatedWith: workout) else {
                    return
                }
                DispatchQueue.main.async {
                    self.heartRateSamplesInDatabase = samples
                }
                print("Loaded \(samples.count) heart rate samples from database")
            } catch {
                print("Failed to load heart rate samples from database: \(error)")
            }
        }
    }

    private func checkPermissionsAndSearchHealth() {
        Task {
            do {
                try await checkPermissionsAndFindWorkout()
            } catch {
                print("Failed to search for workout: \(error)")
            }
        }
    }

    private func checkPermissionsAndFindWorkout() async throws {

        switch  store.authorizationStatus(for: HKWorkout.self) {
        case .notDetermined:
            try await requestWorkoutPermission()
            try await checkPermissionsAndFindWorkout()
        case .sharingAuthorized:
            await findWorkoutInHealth()
        case .sharingDenied:
            print("No permission to write workouts")
            await findWorkoutInHealth()
            return
        @unknown default:
            print("Unknown permission for workouts")
            return
        }
    }

    private func requestWorkoutPermission() async throws {
        try await store.requestAuthorization(read: HKWorkout.self)
    }

    private func findWorkoutInHealth() async {
        guard let activityType = workout.workoutActivities.first?.workoutConfiguration.activityType else {
            print("No activity type to find workout")
            return
        }

        let start = workout.startDate.addingTimeInterval(-60)
        let end = workout.endDate.addingTimeInterval(60)
        guard let workout = try? await store.workouts(activityType: activityType, from: start, to: end)
            .first else {
            print("No workout found or error")
            return
        }

        print("Found matching workout in health")
        DispatchQueue.main.async {
            self.healthWorkout = workout
        }

        do {
            let heartRates: [HeartRate] = try await store.samples(associatedWith: workout)
            print("Found \(heartRates.count) heart rate samples in Health")
            DispatchQueue.main.async {
                self.heartRateSamplesInHealth = heartRates
            }
        } catch {
            print("Failed to get heart rates for workout: \(error)")
        }
    }
}

#Preview {
    return NavigationStack {
        WorkoutDetailView(workout: .mock1)
            .environmentObject(Database.mock)
    }
}

extension String: Identifiable {
    
    public var id: Self { self }
}

private extension Array where Element == HeartRate {

    var minimumHeartRate: Int {
        self.min {
            $0.value < $1.value
        }?.beatsPerMinute ?? 0
    }

    var maximumHeartRate: Int {
        self.max {
            $0.value < $1.value
        }?.beatsPerMinute ?? 0
    }
}

private extension HeartRate {

    var beatsPerMinute: Int {
        quantity.doubleValue(for: .count().unitDivided(by: .minute())).roundedInt
    }
}
